"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.downloadAndExtractNpmModuleAsync = downloadAndExtractNpmModuleAsync;
exports.extractLocalNpmTarballAsync = extractLocalNpmTarballAsync;
exports.extractNpmTarballAsync = extractNpmTarballAsync;
exports.extractNpmTarballFromUrlAsync = extractNpmTarballFromUrlAsync;
exports.getNpmUrlAsync = getNpmUrlAsync;
exports.npmViewAsync = npmViewAsync;
exports.sanitizeNpmPackageName = sanitizeNpmPackageName;

function _spawnAsync() {
  const data = _interopRequireDefault(require("@expo/spawn-async"));

  _spawnAsync = function () {
    return data;
  };

  return data;
}

function _assert() {
  const data = _interopRequireDefault(require("assert"));

  _assert = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function _slugify() {
  const data = _interopRequireDefault(require("slugify"));

  _slugify = function () {
    return data;
  };

  return data;
}

function _stream() {
  const data = require("stream");

  _stream = function () {
    return data;
  };

  return data;
}

function _tar() {
  const data = _interopRequireDefault(require("tar"));

  _tar = function () {
    return data;
  };

  return data;
}

function _util() {
  const data = require("util");

  _util = function () {
    return data;
  };

  return data;
}

function _xdl() {
  const data = require("xdl");

  _xdl = function () {
    return data;
  };

  return data;
}

function _CommandError() {
  const data = _interopRequireDefault(require("../../CommandError"));

  _CommandError = function () {
    return data;
  };

  return data;
}

function _log() {
  const data = _interopRequireDefault(require("../../log"));

  _log = function () {
    return data;
  };

  return data;
}

function _createFileTransform() {
  const data = require("./createFileTransform");

  _createFileTransform = function () {
    return data;
  };

  return data;
}

function _FileSystemCache() {
  const data = require("./fetch-cache/FileSystemCache");

  _FileSystemCache = function () {
    return data;
  };

  return data;
}

function _fetch() {
  const data = _interopRequireDefault(require("./fetch-cache/fetch"));

  _fetch = function () {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const cachedFetch = (0, _fetch().default)(new (_FileSystemCache().FileSystemCache)({
  cacheDirectory: getCacheFilePath() // Time to live. How long (in ms) responses remain cached before being automatically ejected. If undefined, responses are never automatically ejected from the cache.
  // ttl: 1000,

}));

function sanitizeNpmPackageName(name) {
  // https://github.com/npm/validate-npm-package-name/#naming-rules
  return applyKnownNpmPackageNameRules(name) || applyKnownNpmPackageNameRules((0, _slugify().default)(name)) || // If nothing is left use 'app' like we do in Xcode projects.
  'app';
}

function applyKnownNpmPackageNameRules(name) {
  // https://github.com/npm/validate-npm-package-name/#naming-rules
  // package name cannot start with '.' or '_'.
  while (/^(\.|_)/.test(name)) {
    name = name.substring(1);
  }

  name = name.toLowerCase().replace(/[^a-zA-Z0-9._\-/@]/g, '');
  return name // .replace(/![a-z0-9-._~]+/g, '')
  // Remove special characters
  .normalize('NFD').replace(/[\u0300-\u036f]/g, '') || null;
}

async function npmViewAsync(...props) {
  var _await$spawnAsync$std;

  const cmd = ['view', ...props, '--json'];
  const results = (_await$spawnAsync$std = (await (0, _spawnAsync().default)('npm', cmd)).stdout) === null || _await$spawnAsync$std === void 0 ? void 0 : _await$spawnAsync$std.trim();
  const cmdString = `npm ${cmd.join(' ')}`;

  _log().default.debug('Run:', cmdString);

  if (!results) {
    return null;
  }

  try {
    return JSON.parse(results);
  } catch (error) {
    throw new Error(`Could not parse JSON returned from "${cmdString}".\n\n${results}\n\nError: ${error.message}`);
  }
}
/** Given a package name like `expo` or `expo@beta`, return the registry URL if it exists. */


async function getNpmUrlAsync(packageName) {
  const results = await npmViewAsync(packageName, 'dist.tarball');
  (0, _assert().default)(results, `Could not get npm url for package "${packageName}"`); // Fully qualified url returns a string.
  // Example:
  // 𝝠 npm view expo-template-bare-minimum@sdk-33 dist.tarball --json

  if (typeof results === 'string') {
    return results;
  } // When the tag is arbitrary, the tarball url is an array, return the last value as it's the most recent.
  // Example:
  // 𝝠 npm view expo-template-bare-minimum@33 dist.tarball --json


  if (Array.isArray(results)) {
    return results[results.length - 1];
  }

  throw new (_CommandError().default)('Expected results of `npm view ...` to be an array or string. Instead found: ' + results);
} // @ts-ignore


const pipeline = (0, _util().promisify)(_stream().Stream.pipeline);

async function downloadAndExtractNpmModuleAsync(npmName, props) {
  const url = await getNpmUrlAsync(npmName);

  _log().default.debug('Fetch from URL:', url);

  await extractNpmTarballFromUrlAsync(url, props);
}

async function extractLocalNpmTarballAsync(tarFilePath, props) {
  const readStream = _fsExtra().default.createReadStream(tarFilePath);

  await extractNpmTarballAsync(readStream, props);
}

function getCacheFilePath() {
  return _path().default.join(_xdl().UserSettings.dotExpoHomeDirectory(), 'template-cache');
}

async function createUrlStreamAsync(url) {
  const response = await cachedFetch(url);

  if (!response.ok) {
    throw new Error(`Unexpected response: ${response.statusText}. From url: ${url}`);
  }

  return response.body;
}

async function extractNpmTarballFromUrlAsync(url, props) {
  await extractNpmTarballAsync(await createUrlStreamAsync(url), props);
}

async function extractNpmTarballAsync(stream, props) {
  const {
    cwd,
    strip,
    name,
    fileList = []
  } = props;
  await _fsExtra().default.ensureDir(cwd);
  return pipeline(stream, _tar().default.extract({
    cwd,
    transform: (0, _createFileTransform().createFileTransform)(name),
    onentry: (0, _createFileTransform().createEntryResolver)(name),
    strip: strip !== null && strip !== void 0 ? strip : 1
  }, fileList));
}
//# sourceMappingURL=npm.js.map